@@ -29,6 +29,10 @@
 
 #include "lang.h"
 #include <time.h>
+#include <stdlib.h>
+#include <sys/stat.h>
+
+#include "lxcom.h"
 
 enum {
     COL_SESSION_NAME,
@@ -43,22 +47,19 @@ enum {
     N_LANG_COLS
 };
 
-#ifndef VCONFIG_FILE
-#define VCONFIG_FILE "/etc/lxdm/lxdm.conf"
-#endif
+#define VCONFIG_FILE "/var/lib/lxdm/lxdm.conf"
 
-static gboolean config_changed = FALSE;
 static GKeyFile *config;
 static GKeyFile * var_config;
 static GtkWidget* win;
 static GtkWidget* prompt;
 static GtkWidget* login_entry;
-static GtkWidget* prompt;
+static GtkWidget* user_list;
 
 static GtkWidget* sessions;
 static GtkWidget* lang;
 
-static GtkWidget* exit;
+static GtkWidget* exit_btn;
 
 static GtkWidget* exit_menu;
 static GtkWidget *lang_menu;
@@ -69,6 +70,7 @@ static char* session_exec = NULL;
 static char* session_desktop_file = NULL;
 
 static char* ui_file = NULL;
+static char *ui_nobody = NULL;
 
 static GdkPixbuf *bg_img = NULL;
 static GdkColor bg_color = {0};
@@ -90,111 +92,117 @@ static void on_screen_size_changed(GdkScreen* scr, GtkWindow* win)
     gtk_window_resize( win, gdk_screen_get_width(scr), gdk_screen_get_height(scr) );
 }
 
-static void on_entry_activate(GtkEntry* entry, gpointer user_data)
+static void on_entry_activate(GtkEntry* entry)
 {
-    char* tmp;
-    if( !user )
-    {
-        user = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
-        gtk_entry_set_text(GTK_ENTRY(entry), "");
-        gtk_label_set_text( GTK_LABEL(prompt), _("Password:") );
-        if( strchr(user, ' ') )
-        {
-            g_free(user);
-            user = NULL;
-            return;
-        }
-        gtk_entry_set_visibility(entry, FALSE);
-    }
-    else
-    {
-        GtkTreeIter it;
-        char *session_lang = "";
+	char* tmp;
+	if( !user )
+	{
+		user = g_strdup( gtk_entry_get_text( GTK_ENTRY(entry) ) );
+		gtk_entry_set_text(GTK_ENTRY(entry), "");
+		gtk_label_set_text( GTK_LABEL(prompt), _("Password:") );
+		if(strchr(user, ' '))
+		{
+			g_free(user);
+			user = NULL;
+			return;
+		}
+		gtk_entry_set_visibility(entry, FALSE);
+	}
+	else
+	{
+		GtkTreeIter it;
+		char *session_lang = "";
 
-        if( gtk_combo_box_get_active_iter(GTK_COMBO_BOX(sessions), &it) )
-        {
-            GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(sessions) );
-            gtk_tree_model_get(model, &it, 1, &session_exec, 2, &session_desktop_file, -1);
-        }
-        else
-        {
-            /* FIXME: fatal error */
-        }
+		if( gtk_combo_box_get_active_iter(GTK_COMBO_BOX(sessions), &it) )
+		{
+			GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(sessions) );
+			gtk_tree_model_get(model, &it, 1, &session_exec, 2, &session_desktop_file, -1);
+		}
+		else
+		{
+			/* FIXME: fatal error */
+		}
 
-        pass = g_strdup( gtk_entry_get_text(entry) );
-        if( strchr(pass, ' ') )
-        {
-            g_free(user); user = NULL;
-            g_free(pass); pass = NULL;
-            gtk_label_set_text( GTK_LABEL(prompt), _("User:") );
-            gtk_entry_set_text(GTK_ENTRY(entry), "");
-            gtk_entry_set_visibility(GTK_ENTRY(entry), TRUE);
-            return;
-        }
+		tmp = g_strdup( gtk_entry_get_text(entry) );
+		pass=g_base64_encode((guchar*)tmp,strlen(tmp)+1);
+		g_free(tmp);
 
-        if( lang && gtk_combo_box_get_active_iter(GTK_COMBO_BOX(lang), &it) )
-        {
-            GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(lang) );
-            gtk_tree_model_get(model, &it, 1, &session_lang, -1);
-            //FIXME: is session leaked?
-        }
+		if( lang && gtk_combo_box_get_active_iter(GTK_COMBO_BOX(lang), &it) )
+		{
+			GtkTreeModel* model = gtk_combo_box_get_model( GTK_COMBO_BOX(lang) );
+			gtk_tree_model_get(model, &it, 1, &session_lang, -1);
+			//FIXME: is session leaked?
+		}
 
-        tmp = g_key_file_get_string(var_config, "base", "last_session", NULL);
-        if( g_strcmp0(tmp, session_desktop_file) )
-        {
-            g_key_file_set_string(var_config, "base", "last_session", session_desktop_file);
-            config_changed = TRUE;
-        }
-        g_free(tmp);
+		tmp = g_key_file_get_string(var_config, "base", "last_session", NULL);
+		if( g_strcmp0(tmp, session_desktop_file) )
+		{
+			g_key_file_set_string(var_config, "base", "last_session", session_desktop_file);
+		}
+		g_free(tmp);
 
-        tmp = g_key_file_get_string(var_config, "base", "last_lang", NULL);
-        if( g_strcmp0(tmp, session_lang) )
-        {
-            g_key_file_set_string(var_config, "base", "last_lang", session_lang);
-            config_changed = TRUE;
-        }
-        g_free(tmp);
+		tmp = g_key_file_get_string(var_config, "base", "last_lang", NULL);
+		if( g_strcmp0(tmp, session_lang) )
+		{
+			g_key_file_set_string(var_config, "base", "last_lang", session_lang);
+		}
+		g_free(tmp);
 
-        printf("login user=%s pass=%s session=%s lang=%s\n",
-               user, pass, session_exec, session_lang);
+		printf("login user=%s pass=%s session=%s lang=%s\n",
+			user, pass, session_exec, session_lang);
 
-        /* password check failed */
-        g_free(user);
-        user = NULL;
-        g_free(pass);
-        pass = NULL;
+		/* password check failed */
+		g_free(user);
+		user = NULL;
+		g_free(pass);
+		pass = NULL;
 
-        gtk_widget_hide(prompt);
-        gtk_widget_hide( GTK_WIDGET(entry) );
+		gtk_widget_hide(prompt);
+		gtk_widget_hide( GTK_WIDGET(entry) );
 
-        gtk_label_set_text( GTK_LABEL(prompt), _("User:") );
-        gtk_entry_set_text(GTK_ENTRY(entry), "");
-        gtk_entry_set_visibility(GTK_ENTRY(entry), TRUE);
-    }
+		gtk_label_set_text( GTK_LABEL(prompt), _("User:") );
+		gtk_entry_set_text(GTK_ENTRY(entry), "");
+		gtk_entry_set_visibility(GTK_ENTRY(entry), TRUE);
+	}
 }
 
 static void load_sessions()
 {
-    GtkListStore* list;
-    GtkTreeIter it, active_it = {0};
-    char* last;
-    char *path, *file_name, *name, *exec;
-    GKeyFile* kf;
-    GDir* dir = g_dir_open(XSESSIONS_DIR, 0, NULL);
-    if( !dir )
-        return;
+	GtkListStore* list;
+	GtkTreeIter it, active_it = {0};
+	char* last;
+	char *path, *file_name, *name, *exec;
+	GKeyFile* kf;
+	GDir* dir = g_dir_open(XSESSIONS_DIR, 0, NULL);
+	if( !dir )
+		return;
 
-    last = g_key_file_get_string(var_config, "base", "last_session", NULL);
+	last = g_key_file_get_string(var_config, "base", "last_session", NULL);
 
-    list = gtk_list_store_new(N_SESSION_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-    kf = g_key_file_new();
-    while( ( file_name = (char*)g_dir_read_name(dir) ) != NULL )
-    {
-        path = g_build_filename(XSESSIONS_DIR, file_name, NULL);
-        if( g_key_file_load_from_file(kf, path, 0, NULL) )
-        {
-            name = g_key_file_get_locale_string(kf, "Desktop Entry", "Name", NULL, NULL);
-            //exec = g_key_file_get_string(kf, "Desktop Entry", "Exec", NULL);
+	list = gtk_list_store_new(N_SESSION_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	kf = g_key_file_new();
+	while( ( file_name = (char*)g_dir_read_name(dir) ) != NULL )
+	{
+		path = g_build_filename(XSESSIONS_DIR, file_name, NULL);
+		if( g_key_file_load_from_file(kf, path, 0, NULL) )
+		{
+			name = g_key_file_get_locale_string(kf, "Desktop Entry", "Name", NULL, NULL);
+			if(!name || !name[0])
+			{
+				g_free(name);
+				g_free(path);
+				continue;
+			}
+            exec = g_key_file_get_string(kf, "Desktop Entry", "Exec", NULL);
+			if(!exec || !exec[0])
+			{
+				/* bad session config file */
+				g_free(exec);
+				g_free(name);
+				g_free(path);
+				continue;
+			}
+			g_free(exec); /* we just test it, and not use it */
             exec=g_strdup(path);
 
             if( !strcmp(name, "LXDE") )
@@ -205,35 +213,36 @@ static void load_sessions()
                                COL_SESSION_NAME, name,
                                COL_SESSION_EXEC, exec,
                                COL_SESSION_DESKTOP_FILE, file_name, -1);
-
-            if( last && g_strcmp0(file_name, last) == 0 )
+            if( last && g_strcmp0(path, last) == 0 )
+            {
                 active_it = it;
+			}
 
             g_free(name);
             g_free(exec);
         }
         g_free(path);
     }
-    g_dir_close(dir);
-    g_key_file_free(kf);
+	g_dir_close(dir);
+	g_key_file_free(kf);
 
-    gtk_list_store_prepend(list, &it);
-    gtk_list_store_set(list, &it,
-                       COL_SESSION_NAME, _("Default"),
-                       COL_SESSION_EXEC, "",
-                       COL_SESSION_DESKTOP_FILE, "__default__", -1);
-    if( last && g_strcmp0(file_name, last) == 0 )
-        active_it = it;
+	gtk_list_store_prepend(list, &it);
+	gtk_list_store_set(list, &it,
+					   COL_SESSION_NAME, _("Default"),
+					   COL_SESSION_EXEC, "",
+					   COL_SESSION_DESKTOP_FILE, "__default__", -1);
+	if( last && g_strcmp0("__default__", last) == 0 )
+		active_it = it;
 
-    g_free(last);
-    gtk_combo_box_set_model( GTK_COMBO_BOX(sessions), GTK_TREE_MODEL(list) );
-    gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(sessions), 0);
-    if( active_it.stamp )
-        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(sessions), &active_it);
-    else
-        gtk_combo_box_set_active(GTK_COMBO_BOX(sessions), 0);
+	g_free(last);
+	gtk_combo_box_set_model( GTK_COMBO_BOX(sessions), GTK_TREE_MODEL(list) );
+	gtk_combo_box_entry_set_text_column(GTK_COMBO_BOX_ENTRY(sessions), 0);
+	if( active_it.stamp )
+		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(sessions), &active_it);
+	else
+		gtk_combo_box_set_active(GTK_COMBO_BOX(sessions), 0);
 
-    g_object_unref(list);
+	g_object_unref(list);
 }
 
 static void load_lang_cb(void *arg, char *lang, char *desc)
@@ -246,7 +255,6 @@ static void load_lang_cb(void *arg, char *lang, char *desc)
     p=strchr(lang2,'.');
     if(p) *p=0;
 
-    //temp=g_strdup_printf("\xe2\x80\x8e%s\t\t\xe2\x80\xab%s",lang,desc);
     if(lang2[0] && lang2[0]!='~')
         temp=g_strdup_printf("%s\t%s",lang2,desc?desc:"");
     else
@@ -279,7 +287,6 @@ static void on_menu_lang_select(GtkMenuItem *item,gpointer user_data)
 	GtkTreeModel *list;
 	int active=-1;
 	char *temp;
-	GPtrArray *array;
 	if(!sel || !sel[0]) return;
 	
 	list=gtk_combo_box_get_model(GTK_COMBO_BOX(lang));
@@ -307,26 +314,6 @@ static void on_menu_lang_select(GtkMenuItem *item,gpointer user_data)
                        COL_LANG_DISPNAME, temp,
                        COL_LANG, sel, -1);
 	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(lang),&iter);
-
-	array=g_ptr_array_new();
-	res=gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list),&iter);
-	while(res==TRUE)
-	{
-		gtk_tree_model_get(GTK_TREE_MODEL(list),&iter,1,&temp,-1);
-		if(!temp || !temp[0] || temp[0]=='~')
-		{
-			g_free(temp);
-		}
-		else
-		{
-			g_ptr_array_add(array,temp);
-		}
-		res=gtk_tree_model_iter_next(GTK_TREE_MODEL(list),&iter);
-	}
-	g_key_file_set_string_list(var_config,"base","last_langs",(void*)array->pdata,array->len);
-	config_changed=TRUE;
-	g_ptr_array_foreach(array,(GFunc)g_free,0);
-	g_ptr_array_free(array,TRUE);
 }
 
 static void load_menu_lang_cb(void *arg, char *lang, char *desc)
@@ -367,7 +354,7 @@ static void on_lang_changed(GtkComboBox *widget)
 {
 	GtkTreeIter it;
 	if( gtk_combo_box_get_active_iter(widget, &it) )
-        {
+	{
 		GtkListStore *list=(GtkListStore*)gtk_combo_box_get_model(GTK_COMBO_BOX(lang));
 		char *lang=NULL;
 		gtk_tree_model_get(GTK_TREE_MODEL(list), &it, 1, &lang, -1);
@@ -377,12 +364,7 @@ static void on_lang_changed(GtkComboBox *widget)
 			show_all_languages();
 		}
 		g_free(lang);
-        }
-        else
-        {
-            return;
-        }
-
+	}
 }
 
 static void load_langs()
@@ -445,7 +427,7 @@ static void load_exit()
     gtk_menu_shell_append(GTK_MENU_SHELL(exit_menu), item);
 
     gtk_widget_show_all(exit_menu);
-    g_signal_connect(exit, "clicked", G_CALLBACK(on_exit_clicked), NULL);
+    g_signal_connect(exit_btn, "clicked", G_CALLBACK(on_exit_clicked), NULL);
 }
 
 static gboolean on_expose(GtkWidget* widget, GdkEventExpose* evt, gpointer user_data)
@@ -458,7 +440,7 @@ static gboolean on_expose(GtkWidget* widget, GdkEventExpose* evt, gpointer user_
     if( bg_img )
     {
         cairo_matrix_t matrix;
-        double x = -0.5, y = -0.5, sx, sy;
+        double x = 0, y = 0, sx, sy;
         cairo_get_matrix(cr, &matrix);
         sx = (double)gdk_screen_width() / (double)gdk_pixbuf_get_width(bg_img);
         sy = (double)gdk_screen_height() / (double)gdk_pixbuf_get_height(bg_img);
@@ -521,6 +503,130 @@ static gboolean on_timeout(GtkLabel* label)
     return TRUE;
 }
 
+static void on_user_select(GtkIconView *iconview)
+{
+	GList *list=gtk_icon_view_get_selected_items(iconview);
+	GtkTreeIter iter;
+	GtkTreeModel *model=gtk_icon_view_get_model(iconview);
+	char *name;
+	if(!list) return;
+	gtk_tree_model_get_iter(model,&iter,list->data);
+	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free (list);
+	gtk_tree_model_get(model,&iter,2,&name,-1);
+	gtk_widget_hide(user_list);
+	if(name && name[0])
+	{
+		gtk_entry_set_text(GTK_ENTRY(login_entry),name);
+		g_free(name);
+		on_entry_activate(GTK_ENTRY(login_entry));
+		gtk_widget_show(login_entry);
+		gtk_widget_grab_focus(login_entry);
+		
+		gtk_label_set_text( GTK_LABEL(prompt), _("Password:") );
+		gtk_widget_show(prompt);
+	}
+	else
+	{
+		g_free(name);
+		if(user)
+		{
+			g_free(user);
+			user=NULL;
+		}
+		gtk_entry_set_text(GTK_ENTRY(login_entry),"");
+		gtk_widget_show(login_entry);
+		gtk_widget_grab_focus(login_entry);
+		gtk_label_set_text( GTK_LABEL(prompt), _("User:") );
+		gtk_widget_show(prompt);
+	}
+}
+
+static gboolean load_user_list(GtkWidget *widget)
+{
+	GtkListStore *model;
+	GtkTreeIter iter;
+	GKeyFile *kf;
+	char *res=NULL;
+	char **users;
+	gsize count;
+	int i;
+	lxcom_send("/var/run/lxdm/lxdm.sock","USER_LIST",&res);
+	if(!res)
+	{
+		printf("log USER_LIST fail\n");
+		return FALSE;
+	}
+	kf=g_key_file_new();
+	if(!g_key_file_load_from_data(kf,res,-1,0,NULL))
+	{
+		g_key_file_free(kf);
+		g_free(res);
+		printf("log USER_LIST data bad\n");
+		return FALSE;
+	}
+	g_free(res);
+	
+	gtk_icon_view_set_selection_mode(GTK_ICON_VIEW(widget),GTK_SELECTION_SINGLE);
+	gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(widget),0);
+	gtk_icon_view_set_markup_column(GTK_ICON_VIEW(widget),1);
+	gtk_icon_view_set_orientation(GTK_ICON_VIEW(widget),GTK_ORIENTATION_HORIZONTAL);
+	model=gtk_list_store_new(5,GDK_TYPE_PIXBUF,G_TYPE_STRING,
+			G_TYPE_STRING,G_TYPE_STRING,G_TYPE_BOOLEAN);
+	gtk_icon_view_set_model(GTK_ICON_VIEW(widget),GTK_TREE_MODEL(model));
+	g_signal_connect(G_OBJECT(widget),"item-activated",G_CALLBACK(on_user_select),NULL);
+	g_signal_connect(G_OBJECT(widget),"selection-changed",G_CALLBACK(on_user_select),NULL);
+	
+	users=g_key_file_get_groups(kf,&count);
+	if(!users || count<=0)
+	{
+		g_key_file_free(kf);
+		printf("USER_LIST 0 user\n");
+		return FALSE;
+	}
+	if(count>3)
+	{
+		// TODO: better ui needed
+		count=3;
+	}
+	for(i=0;i<count;i++)
+	{		
+		char *gecos,*face_path,*display;
+		gboolean login;
+		GdkPixbuf *face=NULL;
+		gtk_list_store_append(model,&iter);
+		gecos=g_key_file_get_string(kf,users[i],"gecos",0);
+		face_path=g_key_file_get_string(kf,users[i],"face",0);
+		login=g_key_file_get_boolean(kf,users[i],"login",0);
+		if(face_path)
+			face=gdk_pixbuf_new_from_file_at_scale(face_path,48,48,TRUE,NULL);
+		if(!face)
+		{
+			/* TODO: load some default face */
+			face=gdk_pixbuf_new_from_file_at_scale(ui_nobody,48,48,TRUE,NULL);
+		}
+		display=g_strdup_printf("<span font_size=\"x-large\">%s</span>%s%s%s%s",
+			gecos?gecos:users[i],
+			(gecos&&strcmp(gecos,users[i]))?"(":"",
+			(gecos&&strcmp(gecos,users[i]))?users[i]:"",
+			(gecos&&strcmp(gecos,users[i]))?")":"",
+			login?_("\n<i>logged in</i>"):"");
+		// don't translate it now, not freeze
+		gtk_list_store_set(model,&iter,0,face,1,display,2,users[i],3,gecos,4,login,-1);
+		if(face) g_object_unref(G_OBJECT(face));
+		g_free(display);
+		g_free(gecos);
+		g_free(face_path);
+	}
+	g_strfreev(users);
+	g_key_file_free(kf);
+	
+	// add "More ..."
+	gtk_list_store_append(model,&iter);
+	gtk_list_store_set(model,&iter,1,_("More ..."),2,"",3,"",4,FALSE,-1);
+	return TRUE;
+}
+
 static void create_win()
 {
     GtkBuilder* builder;
@@ -551,6 +657,13 @@ static void create_win()
 
     scr = gtk_widget_get_screen(win);
     g_signal_connect(scr, "size-changed", G_CALLBACK(on_screen_size_changed), win);
+    
+    user_list=(GtkWidget*)gtk_builder_get_object(builder,"user_list");
+    if(user_list)
+    {
+		//TODO: load user list, connect select_changed,activate signal
+		// Use GtkIconView may enough
+	}
 
     prompt = (GtkWidget*)gtk_builder_get_object(builder, "prompt");
     login_entry = (GtkWidget*)gtk_builder_get_object(builder, "login_entry");
@@ -587,24 +700,37 @@ static void create_win()
         load_langs();
     }
 
-    if( (w = (GtkWidget*)gtk_builder_get_object(builder, "time"))!=NULL )
-    {
-        guint timeout = g_timeout_add(1000, (GSourceFunc)on_timeout, w);
-        g_signal_connect_swapped(w, "destroy",
-            G_CALLBACK(g_source_remove), GUINT_TO_POINTER(timeout));
-        on_timeout((GtkLabel*)w);
-    }
+	if( (w = (GtkWidget*)gtk_builder_get_object(builder, "time"))!=NULL )
+	{
+		guint timeout = g_timeout_add(1000, (GSourceFunc)on_timeout, w);
+		g_signal_connect_swapped(w, "destroy",
+			G_CALLBACK(g_source_remove), GUINT_TO_POINTER(timeout));
+		on_timeout((GtkLabel*)w);
+	}
 
-    exit = (GtkWidget*)gtk_builder_get_object(builder, "exit");
-    load_exit();
+	exit_btn = (GtkWidget*)gtk_builder_get_object(builder, "exit");
+	load_exit();
 
-    g_object_unref(builder);
+	g_object_unref(builder);
 
-    gtk_window_set_default_size( GTK_WINDOW(win), gdk_screen_get_width(scr), gdk_screen_get_height(scr) );
-    gtk_window_present( GTK_WINDOW(win) );
-    gtk_widget_realize(login_entry);
-    //gdk_keyboard_grab(login_entry->window,FALSE,GDK_CURRENT_TIME);
-    gtk_widget_grab_focus(login_entry);
+	gtk_window_set_default_size( GTK_WINDOW(win), gdk_screen_get_width(scr), gdk_screen_get_height(scr) );
+	gtk_window_present( GTK_WINDOW(win) );
+	gtk_widget_realize(login_entry);
+    
+	if(user_list && !g_key_file_get_integer(config,"userlist","disable",NULL) && 
+			load_user_list(user_list))
+	{
+		gtk_widget_hide(login_entry);
+	}
+	else
+	{
+		if(user_list)
+		{
+			gtk_widget_hide(user_list);
+			user_list=NULL;
+		}
+		gtk_widget_grab_focus(login_entry);
+	}
 }
 
 int set_background(void)
@@ -651,7 +777,6 @@ int set_background(void)
         if( bg[0] == '#' )
             gdk_color_parse(bg, &bg_color);
     }
-        //gdk_window_set_background(win,&screen);
     g_free(bg);
     g_free(style);
     return 0;
@@ -659,25 +784,45 @@ int set_background(void)
 
 static gboolean on_lxdm_command(GIOChannel *source, GIOCondition condition, gpointer data)
 {
-    GIOStatus ret;
-    char *str;
+	GIOStatus ret;
+	char *str;
 
-    if( !(G_IO_IN & condition) )
-        return FALSE;
-    ret = g_io_channel_read_line(source, &str, NULL, NULL, NULL);
-    if( ret != G_IO_STATUS_NORMAL )
-        return FALSE;
+	if( !(G_IO_IN & condition) )
+		return FALSE;
+	ret = g_io_channel_read_line(source, &str, NULL, NULL, NULL);
+	if( ret != G_IO_STATUS_NORMAL )
+		return FALSE;
 
-    if( !strncmp(str, "quit", 4) || !strncmp(str, "exit",4))
-        gtk_main_quit();
-    else if( !strncmp(str, "reset", 5) )
-    {
-        gtk_widget_show(prompt);
-        gtk_widget_show(login_entry);
-        gtk_widget_grab_focus(login_entry);
-    }
-    g_free(str);
-    return TRUE;
+	if( !strncmp(str, "quit", 4) || !strncmp(str, "exit",4))
+	gtk_main_quit();
+	else if( !strncmp(str, "reset", 5) )
+	{
+		if(user)
+		{
+			g_free(user);
+			user=NULL;
+		}
+		if(pass)
+		{
+			g_free(pass);
+			pass=NULL;
+		}
+		gtk_label_set_text( GTK_LABEL(prompt), _("User:"));
+		gtk_widget_show(prompt);
+		if(user_list)
+		{
+			gtk_widget_hide(login_entry);
+			gtk_icon_view_unselect_all(GTK_ICON_VIEW(user_list));
+			gtk_widget_show(user_list);
+		}
+		else
+		{
+			gtk_widget_show(login_entry);
+			gtk_widget_grab_focus(login_entry);
+		}
+	}
+	g_free(str);
+	return TRUE;
 }
 
 void listen_stdin(void)
@@ -719,7 +864,6 @@ static void apply_theme(const char* theme_name)
 
     if( g_file_test(rc, G_FILE_TEST_EXISTS) )
     {
-        g_debug("%s", rc);
         gtk_rc_parse(rc);
     }
     g_free(rc);
@@ -729,12 +873,26 @@ static void apply_theme(const char* theme_name)
         g_free(ui_file);
         ui_file = NULL;
     }
+    
+    ui_nobody = g_build_filename(theme_dir, "nobody.png", NULL);
+    if( !g_file_test(ui_nobody, G_FILE_TEST_EXISTS) )
+    {
+        g_free(ui_nobody);
+        ui_nobody = NULL;
+    }
+    
     g_free(theme_dir);
+    
+    
 }
 
 int main(int arc, char *arg[])
 {
     char* theme_name;
+    GtkSettings*p;
+
+    /* this will override LC_MESSAGES */
+    unsetenv("LANGUAGE");
 
     gtk_set_locale();
     bindtextdomain("lxdm", "/usr/share/locale");
@@ -749,6 +907,14 @@ int main(int arc, char *arg[])
 
     gtk_init(&arc, &arg);
 
+    p=gtk_settings_get_default();
+    if(p)
+    {
+        setenv("GTK_IM_MODULE","gtk-im-context-simple",1);
+        gtk_settings_set_string_property(p,"gtk-im-module","gtk-im-context-simple",0);
+        gtk_settings_set_long_property(p,"gtk-show-input-method-menu",0,0);
+    }
+
     set_background();
     set_root_background();
 
@@ -778,13 +944,6 @@ int main(int arc, char *arg[])
 
     gtk_main();
 
-    if( config_changed )
-    {
-        gsize len;
-        char* data = g_key_file_to_data(var_config, &len, NULL);
-        g_file_set_contents(VCONFIG_FILE, data, len, NULL);
-        g_free(data);
-    }
     g_key_file_free(config);
     g_key_file_free(var_config);
 
